<?php
namespace App\Controller;

use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;

use App\Templates;
use App\Exception;

class OrderController {
    
    const FAILOVER_EMAIL = "delivery@teknisa.com";  

    private $integrationService;
    private $connection;
    private $util;
    
    public function __construct (\Doctrine\DBAL\Connection $connection,
                                 \App\Service\IntegrationService $integrationService,
                                 \App\Service\ConsumerService $consumerService,
                                 \App\Util\Util $util) {

        $this->connection         = $connection;
        $this->integrationService = $integrationService;
        $this->consumerService    = $consumerService;
        $this->util               = $util;
    }
    
    public function welcome(Request $request, Response $response) {
        global $CLIENT_DATA;
        
        $config      = $CLIENT_DATA;
        $sql         = "SELECT CDFILIAL FROM FILIAL";
        $result      = $this->connection->fetchAll($sql);
        $versionInfo = $this->getVersionInfo();

        return $response->withJson([
            'client'       => $config['client'],
            'version'      => $versionInfo['version'],
            'revision'     => $versionInfo['revision'],
            'date'         => $versionInfo['date'],
            'filialCount'  => count($result)
        ]);
    }

    public function runWebService (Request $request, Response $response) {
        global $CLIENT_DATA;

        $fileName = "";
        $rawOrder = "";

        try {
            $config = $CLIENT_DATA;
            $order  = $request->getParsedBody();

            if(empty($order)) {
                throw new \Exception("Empty order body");
            }

            $rawOrder = $order;
            
            // ----- Rule CPF In Note
            if(!empty($order['cpfInNote']) && $this->util->isValidCPF($order['cpfInNote'])){
                $order['cpfInNote'] = preg_replace( '/[^0-9]/is', '', $order['cpfInNote']);
            }else{
                $order['cpfInNote'] = null;
            }

            // ----- Rule CPF
            if(!empty($order['consumerDetail']['cpf']) && $this->util->isValidCPF($order['consumerDetail']['cpf'])){
                $order['consumerDetail']['cpf'] = preg_replace( '/[^0-9]/is', '', $order['consumerDetail']['cpf']);
            }else if(!empty($order['consumerDetail']['cpf']) && !$this->util->isValidCPF($order['consumerDetail']['cpf'])) {
                throw new Exception\HTTPBodyException("O CPF informado no pedido é inválido.");
            }
            
            $this->util->validadeRequestBody($order);
            
            $this->mailList  = $this->util->getMailList($order['origin']);
            
            $this->connection->beginTransaction();
            
            $fileName = ((!empty($order["subsidiaryCode"])) && (!empty($order["orderNumber"]))) ? "{$order['subsidiaryCode']}-{$order['orderNumber']}-{$this->util->getCurrentDateTime()}.json" : ("INDEFINIDO-" . $this->util->getCurrentDateTime()) . ".json";
            
            $this->util->writeLog('order', $fileName, $order);
            $useOrderStatus = $config["useOrderStatus"];
            $comd = $this->processOrder($order, $useOrderStatus);
            $idParts = explode ("-",$comd);
             
            $this->integrationService->validateOrder($idParts[0],$idParts[2]);
            $this->connection->commit();
            $this->util->writeLog('order_processed', $fileName, $order);

            return $response->withJson([
                'success' => true,
                'message' => empty($order["scheduleDate"]) ? "Pedido integrado com sucesso." : "Pedido agendado com sucesso.",
                'result'  => ['Order' => $comd]
            ]);
        } catch (Exception\ParamsException $e) {
            $this->connection->rollback();
            
            $msg = date("d-m-Y H:i:s") . " - " . $e->getMessage();
            $this->util->writeLog('order_error', $fileName, $msg);

            $this->util->sendMail((!empty($this->mailList) ? $this->mailList : [self::FAILOVER_EMAIL]), [], "Order API", Templates\defaultMail::TEMPLATE, [
                ":NUMERO_PEDIDO" => "#".$order["subsidiaryCode"]."-".$order["orderNumber"],
                ":NOME_CLIENTE"  => $config["client"],
                ":NOME_SERVIDOR" => $config["server"],
                ":ERRO_MSG"      => $e->getMessage(),
                ":JSON_PEDIDO"   => json_encode($rawOrder, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE),
                ":GERADO_EM"     => date("d/m/Y H:i:s")
            ]);
            
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]);
        } catch (Exception\HTTPBodyException $e) {
            $msg = date("d-m-Y H:i:s") . " - " . $e->getMessage();

            $fileName = ("BODY-EXCEPTION-" . $this->util->getCurrentDateTime()) . ".json";

            $this->util->writeLog('order_error', $fileName, $msg);
            $this->util->sendMail((!empty($this->mailList) ? $this->mailList : [self::FAILOVER_EMAIL]), [], "Order API", Templates\defaultMail::TEMPLATE, [
                ":NUMERO_PEDIDO" => "Indefinido",
                ":NOME_CLIENTE"  => $config["client"],
                ":NOME_SERVIDOR" => $config["server"],
                ":ERRO_MSG"      => $e->getMessage(),
                ":JSON_PEDIDO"   => json_encode($rawOrder, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE),
                ":GERADO_EM"     => date("d/m/Y H:i:s")
            ]);
            
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]);
        } catch (Exception\GeneralErrorException $e) {

            $this->util->sendMail((!empty($this->mailList) ? $this->mailList : [self::FAILOVER_EMAIL]), [], "Order API", Templates\defaultMail::TEMPLATE, [
                ":NUMERO_PEDIDO" => "Indefinido",
                ":NOME_CLIENTE"  => $config["client"],
                ":NOME_SERVIDOR" => $config["server"],
                ":ERRO_MSG"      => $e->getMessage(),
                ":JSON_PEDIDO"   => json_encode($rawOrder, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE),
                ":GERADO_EM"     => date("d/m/Y H:i:s")
            ]);

            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        } catch (\Exception $e) {
            $msg        = "Erro: {$e->getMessage()} \n";
            $fileName   = "INDEFINIDO-" . $this->util->getCurrentDateTime() . ".json";

            $this->util->writeLog('order_error', $fileName, $msg);
            
            $this->util->sendMail((!empty($this->mailList) ? $this->mailList : [self::FAILOVER_EMAIL]), [], "Order API", Templates\defaultMail::TEMPLATE, [
                ":NUMERO_PEDIDO" => "Indefinido",
                ":NOME_CLIENTE"  => $config["client"],
                ":NOME_SERVIDOR" => $config["server"],
                ":ERRO_MSG"      => $e->getMessage(),
                ":JSON_PEDIDO"   => json_encode($rawOrder, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE),
                ":GERADO_EM"     => date("d/m/Y H:i:s")
            ]);

            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }

    public function orderStatus(Request $request, Response $response) {
        $params  = $request->getParsedBody();
        $orderId = $params['orderId'];
        
        try{
            return $response->withJson([
                'orderStatus' => $this->integrationService->getOrderStatus($orderId)
            ]);
        } catch (\Exception $e) {
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }

    public function orderCancel(Request $request, Response $response) {
        $params  = $request->getParsedBody();
        $orderId = $params['orderId'];
        
        try {
            return $response->withJson([
                'success' => $this->integrationService->orderCancel($orderId)
            ]);
        } catch (\Exception $e) {
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }

    private function processOrder ($orderData, $useOrderStatus = false) {
        $generalParams         = $this->integrationService->generalParameters($orderData["subsidiaryCode"]);
        $generalParams["TOGO"] = ($orderData["orderType"] == 2) ? true : false;
        
        if(($orderData["orderType"] == 2) && ($orderData["tax"] != 0)) {
            throw new Exception\ParamsException("Não é possível enviar valores de taxa de entrega para a modalidade TO GO (orderType 2).");
        }
        
        $consumerFields = array_filter($orderData["consumerDetail"], function ($assocIndex) {
            return in_array($assocIndex, $this->consumerFieldsList());
        }, ARRAY_FILTER_USE_KEY);
        
        $addressDetails = !empty($orderData['consumerDetail']['address']) ? $orderData['consumerDetail']['address'] : null;
        
        if (array_key_exists("consumerRef",$orderData["consumerDetail"])){
            $consumerFields["consumerRef"] = $orderData["consumerDetail"]["consumerRef"];
        }
        $consumerDetails = $this->integrationService->fetchConsumerDetails($consumerFields, $addressDetails, $generalParams);

        if(!empty($consumerDetails['SGESTADO'])) {
            $generalParams['SGESTADO'] = $consumerDetails['SGESTADO'];
        }

        return $this->integrationService->openOrderBill($orderData, $generalParams, $consumerDetails, $useOrderStatus);
    }
    
    private function consumerFieldsList () {
        return array(
            "clientId",
            "name",
            "email",
            "phone",
            "cpf",
        );
    }
    
    private function addressFieldsList () {
        return array(
            "zipCode",
            "street",
            "number",
            "neighborhood",
            "city",
            "uf",
            "complement",
            "referencePoint"
        );
    }
    
    private function getVersionInfo() {
        $file = file_get_contents(__DIR__ . "/../../version.json");
        return json_decode($file, true);
    }
    
    public function setSalesOrigin() {
        $date = date("d-m-Y H:i:s");
        $fileName = 'salesOriginLog.txt';
        
        $getServiceInit = round(microtime(true) * 1000);
        
        $this->util->writeLog('salesOrigin', $fileName, $date." - Iniciando atualização de origem de vendas.\n\n");
        
        $salesData = $this->integrationService->getSalesWithoutOrigin();
        
        $this->util->writeLog('salesOrigin', $fileName, count($salesData)." Vendas sem origem encontradas.\n\n");
        
        $this->integrationService->setSalesOrigin($salesData);
        $this->util->writeLog('salesOrigin', $fileName, "Vendas atualizadas.\n\n");
        
        $getServiceEnd = round(microtime(true) * 1000);
        $this->util->writeLog('salesOrigin', $fileName, "Execução finalizada apos ".($getServiceEnd - $getServiceInit)." ms.\n\n\n\n----------||----------\n\n\n\n");
    }
    public function getConsumerById(Request $request, Response $response) {
        try{
            $params = $request->getParsedBody();
            $cosumerIds = $this->consumerService->getIdConsumer($params);
            return $response->withJson($cosumerIds);
            
        } 
        catch (\Exception $e) {
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }
    
    public function exportOrderHistory(Request $request, Response $response){
        try{
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Iniciando rotina de exportacao de status de pedidos.\n");
            
            $params = $request->getParsedBody();
            
            $cdSubProj = $this->integrationService->getCdSubProj($params['origin']);
            
            $url = $this->integrationService->getSubProjectUrlOrders($params['origin']);
            
            $orderHistory = $this->integrationService->getOrderHistory($cdSubProj);
            
            foreach($orderHistory as $order){
                $exportArray = [
                    'orderId' => $order['CDFILIAL'] . '-' . $order['NRVENDAREST'] . '-' . $order['NRCOMANDA'] . '-' . $order['NRCOMANDAEXT'],
                    'orderType'=> ($order['IDRETBALLOJA'] === 'N') ? 'DELIVERY' : 'TOGO',
                    'origin' => $params['origin'],
                    'status' => $this->getOrderHistStatus($order['IDHISTPEDCONS']),
                    'eventData' => $order['DTHRHISTPEDCONS']
                ];
                $exportArray['eventData'] = '20' . substr($exportArray['eventData'], 6, 2) . '-' . substr($exportArray['eventData'], 3, 2) . '-' . substr($exportArray['eventData'], 0, 2) . 'T00:00:00Z';
                
                $header = array(
                    'Content-Type: application/json',
                    'X-WE-API-KEY: TIv6e3m6F0zfQ2Skk03HQaek2ON3z5cB'
                );
                
                $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Processando exportacao de status do pedido {$exportArray['orderId']} - {$exportArray['status']}. \n");
                
                $curl = curl_init();
                
                curl_setopt_array($curl, array(
                        CURLOPT_URL => $url,
                        CURLOPT_RETURNTRANSFER => true,
                        CURLOPT_ENCODING => '',
                        CURLOPT_MAXREDIRS => 10,
                        CURLOPT_TIMEOUT => 15,
                        CURLOPT_FOLLOWLOCATION => true,
                        CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                        CURLOPT_CUSTOMREQUEST => 'POST',
                        CURLOPT_POSTFIELDS => json_encode($exportArray),
                        CURLOPT_HTTPHEADER => $header,
                        CURLOPT_SSL_VERIFYPEER => false
                    )
                );
                
                $response = curl_exec($curl);
                
                $httpCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
                $curlError = curl_error($curl);
                
                curl_close($curl);
                
                if (!empty($curlError) || ($httpCode != 200) || (stripos($response, "<html>")) || (stripos($response, "Fatal error"))) {
                    $fileName = "EXP-".date("H-i-s") . ".txt";
        
                    $this->util->writeLog('network', $fileName, $this->util->formatNetworkError([
                        'eventTime' => date("d/m/Y H:i:s"),
                        'header'    => $header,
                        'url'       => $url,
                        'method'    => 'POST',
                        'params'    => json_encode($exportArray),
                        'httpCode'  => $httpCode,
                        'error'     => $curlError,
                        'response'  => $response
                    ]));

                    $this->integrationService->insertOrderHistControl($order, 'N', $response, $cdSubProj);
                    
                    $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro na exportacao de status do pedido {$exportArray['orderId']} - {$exportArray['status']}. \n");
                } else{
                    $this->integrationService->insertOrderHistControl($order,  'S', $response, $cdSubProj);
                    $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Exportacao de status do pedido {$exportArray['orderId']} - {$exportArray['status']} finalizada com sucesso. \n");
                }
            }
            
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Rotina de exportacao de status de pedidos executada com sucesso. \n");
            
            return json_encode([
                'success' => true,
                'message' => 'Rotina de exportacao de status de pedidos executada com sucesso',
                'result'  => []
            ]);
        } catch(\Exception $e){
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Rcorreu um erro na rotina de exportacao de status de pedidos. \n");
            
            return json_encode([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]);
        }
    }
    
    public function exportSalesNfceAcess(Request $request, Response $response){
        try{
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Iniciando rotina de exportacao de chaves de acesso de vendas. \n"); 
            
            $params = $request->getParsedBody();
            
            $cdSubProj = $this->integrationService->getCdSubProj($params['origin']);
            
            $url = $this->integrationService->getSubProjectUrlSales($params['origin']);
            
            $sales = $this->integrationService->getSalesWithNfceAccess($cdSubProj);

            foreach($sales as $sale){
                $exportArray = [
                    'orderId' => $sale['CDFILIAL'] . '-' . $sale['NRVENDAREST'] . '-' . $sale['NRCOMANDA'] . '-' . $sale['NRCOMANDAEXT'],
                    'orderType'=> ($sale['IDRETBALLOJA'] === 'N') ? 'DELIVERY' : 'TOGO',
                    'origin' => $params['origin'],
                    'accessKey' => $sale['NRACESSONFCE'],
                    'saleData' => $sale['DTENVIONFCE']
                ];
                $exportArray['saleData'] = '20' . substr($exportArray['saleData'], 6, 2) . '-' . substr($exportArray['saleData'], 3, 2) . '-' . substr($exportArray['saleData'], 0, 2) . 'T00:00:00Z';
                
                $header = array(
                    'Content-Type: application/json',
                    'X-WE-API-KEY: TIv6e3m6F0zfQ2Skk03HQaek2ON3z5cB'
                );
                
                $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Processando exportacao de chave de acesso da venda {$exportArray['orderId']} - {$exportArray['accessKey']}. \n");
                
                $curl = curl_init();
                
                curl_setopt_array($curl, array(
                        CURLOPT_URL => $url,
                        CURLOPT_RETURNTRANSFER => true,
                        CURLOPT_ENCODING => '',
                        CURLOPT_MAXREDIRS => 10,
                        CURLOPT_TIMEOUT => 15,
                        CURLOPT_FOLLOWLOCATION => true,
                        CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                        CURLOPT_CUSTOMREQUEST => 'POST',
                        CURLOPT_POSTFIELDS => json_encode($exportArray),
                        CURLOPT_HTTPHEADER => $header,
                        CURLOPT_SSL_VERIFYPEER => false
                    )
                );
                
                $response = curl_exec($curl);
                
                $httpCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
                $curlError = curl_error($curl);
                
                curl_close($curl);
                
                if (!empty($curlError) || ($httpCode != 200) || (stripos($response, "<html>")) || (stripos($response, "Fatal error"))) {
                    $fileName = "EXP-".date("H-i-s") . ".txt";
        
                    $this->util->writeLog('network', $fileName, $this->util->formatNetworkError([
                        'eventTime' => date("d/m/Y H:i:s"),
                        'header'    => $header,
                        'url'       => $url,
                        'method'    => 'POST',
                        'params'    => json_encode($exportArray),
                        'httpCode'  => $httpCode,
                        'error'     => $curlError,
                        'response'  => $response
                    ]));

                    $this->integrationService->insertSaleAcessNfceControl($sale, 'N', $response, $cdSubProj);
                    
                    $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro na exportacao da chave de acesso da venda {$exportArray['orderId']} - {$exportArray['accessKey']}. \n");
                } else{
                    $this->integrationService->insertSaleAcessNfceControl($sale, 'S', $response, $cdSubProj);
                    $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Exportacao da chave de acesso da venda {$exportArray['orderId']} - {$exportArray['accessKey']} finalizada com sucesso. \n");
                }
            }
            
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Rotina de exportacao de chaves de acesso de vendas executada com sucesso. \n");
            
            return json_encode([
                'success' => true,
                'message' => 'Rotina de exportacao de chaves de acesso de vendas executada com sucesso',
                'result'  => []
            ]);
        } catch(\Exception $e){
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Ocorreu um erro na rotina de exportacao de chaves de acesso de vendas. \n");
            
            return json_encode([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]);
        }
    }
    
    private function getOrderHistStatus($status){
        if($status === 'R') return 'STATUS_ACCEPTED';
        else if($status === 'P') return 'STATUS_PREPARATION';
        else if($status === 'S' || $status === 'F') return 'STATUS_FINISHED';
        else if($status === 'E') return 'STATUS_DELIVERED';
        else if($status === 'C') return 'STATUS_CANCELLED';
    }
}
