<?php

namespace App\Service;

use App\Exception;

class IntegrationService {

    private $util;
    private $consumerService;
    private $productService;
    private $paymentFactory;
    private $consumerFactory;
    private $storeFactory;
    private $orderFactory;
    private $productFactory;
    private $branchFactory;
    
    public function __construct (\App\Util\Util $util, 
                                 \App\Service\ConsumerService $consumerService,
                                 \App\Service\ProductService $productService, 
                                 \App\Factory\PaymentFactory $paymentFactory, 
                                 \App\Factory\ConsumerFactory $consumerFactory, 
                                 \App\Factory\StoreFactory $storeFactory,
                                 \App\Factory\OrderFactory $orderFactory, 
                                 \App\Factory\ProductFactory $productFactory,
                                 \App\Factory\BranchFactory $branchFactory) {
        $this->util = $util;
        
        $this->consumerService = $consumerService;
        $this->productService  = $productService;
        $this->paymentFactory  = $paymentFactory;
        $this->consumerFactory = $consumerFactory;
        $this->storeFactory    = $storeFactory;
        $this->orderFactory    = $orderFactory;
        $this->productFactory  = $productFactory;
        $this->branchFactory   = $branchFactory;
    }
    
    public function generalParameters ($branch) {
        $vendor        = $this->storeFactory->getStandardVendor($branch);
        $store         = $this->storeFactory->getStandardStore($branch, $vendor);
        $cashier       = $this->storeFactory->getStandardCashier($branch, $vendor);
        $idControlProd = $this->storeFactory->getProductControlId($branch, $store);
        
        return array(
            "CDFILIAL"     => $branch,
            "CDVENDEDOR"   => $vendor,
            "CDLOJA"       => $store,
            "CDCAIXA"      => $cashier["CDCAIXA"],
            "CDOPERADOR"   => $cashier["CDOPERADOR"],
            "IDCONTROPROD" => $idControlProd
        );
    }
    
    public function fetchConsumerDetails($consumerDetails, $addressDetails, $generalParams) {

	    $consumer          = [];
        $branchAddressInfo = $this->branchFactory->getAddressInfo($generalParams["CDFILIAL"]);
        $address           = $this->processAddressInformation($addressDetails, $branchAddressInfo, $generalParams);
        if (empty($consumerDetails['consumerRef'])) {    
            $standardClient    = $this->branchFactory->getStandardClient($generalParams["CDFILIAL"])['CDCLIENTE'];
            
            if (!empty($consumerDetails["email"])) {
                $consumer = $this->consumerFactory->getConsumerByEmail($consumerDetails["email"], $standardClient);
            }
            
            if (empty($consumer) && !empty($consumerDetails["phone"])) {
                $phone = preg_replace("/[^0-9]/", "", $consumerDetails["phone"]);
                if (!empty($phone)) {
                    $consumer = $this->consumerFactory->getConsumerByPhone($phone, $standardClient);
                }
            }
            
            if (empty($consumer) && !empty($consumerDetails["cpf"])) {
                $consumer = $this->consumerFactory->getConsumerByCPF($consumerDetails["cpf"], $standardClient);
            }
    
            if (empty($consumer)) {
                $addressDetails = empty($addressDetails) ? [] : $addressDetails;
                $consumer = $this->consumerService->registerConsumer($consumerDetails, array_merge($addressDetails, $address), $generalParams);
            }
            
        }
        else {
             $consumer = $this->consumerFactory->getConsumerByIdConsumer($consumerDetails['consumerRef']);
        }
        
        if(empty($consumer)) {
                throw new Exception\ParamsException("Erro na validação de consumidor.");
            }
            
        return array_merge($consumer, $address);
    }
    
    private function processAddressInformation ($addressDetails, $branchAddressInfo, $generalParams) {
        $consumer = [];
        
        $cityCode = $this->consumerFactory->getCityCode($addressDetails["uf"], $addressDetails["city"]);
        
        $consumer["cep"]            = !empty ($addressDetails["zipCode"]) ? preg_replace("/[^0-9]/", "", $addressDetails["zipCode"]) : preg_replace("/[^0-9]/", "", $branchAddressInfo["NRCEPFILI"]);
        $consumer["street"]         = !empty ($addressDetails["street"]) ? $addressDetails["street"] : $branchAddressInfo["DSENDEFILI"];
        $consumer["number"]         = !empty ($addressDetails["number"]) ? $addressDetails["number"] : $branchAddressInfo["NRENDEFILISEP"];
        $consumer["neighborhood"]   = !empty ($addressDetails["neighborhood"]) ? $addressDetails["neighborhood"] : $branchAddressInfo["NMBAIRFILI"];
        $consumer["SGESTADO"]       = !empty ($addressDetails["uf"]) ? $addressDetails["uf"] : $branchAddressInfo["SGESTADO"];
        $consumer["CDMUNICIPIO"]    = !empty ($cityCode) ? $cityCode : $branchAddressInfo["CDMUNICIPIO"];
        $consumer["city"]           = $this->branchFactory->getCityName($consumer["SGESTADO"], $consumer["CDMUNICIPIO"]);
        $consumer["CDBAIRRO"]       = $this->consumerFactory->getNeighborhoodCode($consumer);
        $consumer["complement"]     = $addressDetails["complement"];
        $consumer["referencePoint"] = $addressDetails["referencePoint"];

        return $consumer;
    }
    
    private function checkForMoneyReceipt ($change, $payments, $receiptsArray) {
        $moneyCode = $this->paymentFactory->getMoneyReceiptCode();
        
        if ($change > 0 && !in_array($moneyCode, $receiptsArray))
            throw new \Exception("Troco foi solicitado, mas nenhum tipo de pagamento do tipo Dinheiro foi enviado.");
            
        $moneyMoviment = array_filter($payments, function($payment) use ($moneyCode) {
            $receiptType = $this->paymentFactory->getReceiptType($payment["paymentId"]);
            return $receiptType == $moneyCode;
        });
        
        if (count($moneyMoviment) > 1)
            throw new \Exception("Não é permitido o envio do tipo de pagamento Dinheiro mais de uma vez.");
        
        return $moneyMoviment[0];
    }
    
    public function validateOrder($cdFilial, $nrVendaRest) {
        return $this->orderFactory->validateOrder($cdFilial, $nrVendaRest);
    }
    
    public function openOrderBill ($orderData, $generalParams, $consumerDetails, $useOrderStatus = false) {
        $sale = $this->util->newCode("VENDAREST".$generalParams["CDFILIAL"], 10);
        $comd = $this->util->newCode("COMANDAVEN".$generalParams["CDFILIAL"], 10);
        $isAutomaticPrint = $this->storeFactory->isPosEnabledToAutomaticOrderPrinting($generalParams["CDFILIAL"], $generalParams["CDLOJA"], $generalParams["CDCAIXA"]);
        $tableNumber = !empty($orderData['tableNumber']) ? $orderData['tableNumber'] : "";
        if(!empty($tableNumber))
        $this->orderFactory->validateTableNumber($generalParams['CDFILIAL'],$generalParams['CDLOJA'], $tableNumber);
        $this->orderFactory->insertSale($sale, $generalParams, $consumerDetails, $tableNumber);
        if(!empty($isAutomaticPrint)){
             $orderData["IDIMPAUTCUPEXP"] = $isAutomaticPrint["IDIMPAUTCUPEXP"];
             $orderData["IDIMPAUTCUPFOS"] = $isAutomaticPrint["IDIMPAUTCUPFOS"];
        }
       
        $orderData["NRVENDAREST"] = $sale;
        $orderData["NRCOMANDA"]   = $comd;
        $orderData["sourceId"]    = "DLV_ORD";
        
         if(!empty($orderData["scheduleDate"])) {
            $utcTimezone   = new \DateTimeZone("UTC");
            $orderDateTime = new \DateTime($orderData["scheduleDate"], $utcTimezone);
            $storeTimezone = new \DateTimeZone($this->util->getCurrentTimezone($generalParams["SGESTADO"]));
            
            $orderDateTime->setTimezone($storeTimezone);
  
            $today = date("Y/m/d H:i:s");
            
            $diff  = (strtotime($orderData["scheduleDate"]) - strtotime($today));
            
            if(($diff/60) < 30)
                throw new \Exception("É necessário agendar o pedido com no mínimo 30 minutos de antecedência, escolha um horário para daqui 30 minutos no minimo");

            $orderData["DTHRAGENDADA"] = date_format($orderDateTime, "d/m/Y H:i:s");
            $orderData["IDSTCOMANDA"]  = "A";
            $orderData["IDSINCAGENDA"] = "N";
        } else {
            $orderData["DTHRAGENDADA"] = null;
            $orderData["IDSTCOMANDA"]  = $generalParams["IDCONTROPROD"] === "S" ? "1" : "3";
            $orderData["IDSINCAGENDA"] = "S";
        }
        
         if (($orderData["IDIMPAUTCUPEXP"] == "S") && ($orderData["IDIMPAUTCUPFOS"] == "S"))  {
            $orderData["IDIMPCUPEXP"] = 2;
        } else { 
            $orderData["IDIMPCUPEXP"] = 0;
        }
    
        $this->orderFactory->insertSaleCommand($orderData, $generalParams, $consumerDetails);
        
        if($useOrderStatus){
            $this->orderFactory->insertHistPedCons($orderData, $generalParams);
        }

        $MOVCAIXADLV_SEQ = 1;
        
        $receiptsArray         = [];
        $moneyCode             = $this->paymentFactory->getMoneyReceiptCode();
       
        foreach ($orderData["payments"] as $payment) {
            if (empty($payment["value"]) || ($payment["value"] < 0.01) )
                throw new Exception\ParamsException("O pagamento ".$payment["paymentId"]." (R$".$payment["value"]. ") não atende o valor mínimo (R$ 0,01).");
                
            $receiptType = $this->paymentFactory->getReceiptType($payment["paymentId"]);
            $personalCreditReceipt = $this->paymentFactory->getPersonalCreditReceiptCode($receiptType);

            if (!empty( $personalCreditReceipt)){
                $validateCredit = $this->validatePersonalCredit($orderData,$consumerDetails);
                
                if (!empty($validateCredit)) {
                    throw new Exception\ParamsException('O consumidor ' . $consumerDetails["NMCONSUMIDOR"] . ' não possui saldo de crédito suficiente na(s) familia(s) de produtos para finalizacao deste pedido (familia(s): '.$validateCredit.')');
                }
            }
            array_push($receiptsArray, $receiptType);
            
            if ($receiptType == $moneyCode)
                $payment["value"] = empty($orderData["troco"]) ? $payment["value"] : $orderData["troco"];
            
            $orderData["NRSEQMOVDLV"] = str_pad((string) ($MOVCAIXADLV_SEQ), 10, '0', STR_PAD_LEFT);
            $MOVCAIXADLV_SEQ++;

            $orderData["CDTIPORECE"]  = $receiptType;
            $this->orderFactory->insertCashMoviment($orderData, $payment, $generalParams, $consumerDetails);
        }
        
        if ($orderData["troco"] > 0) {
            $moneyReceipt = $this->checkForMoneyReceipt($orderData["troco"], $orderData["payments"], $receiptsArray);

            $movCaixa = str_pad((string) ($MOVCAIXADLV_SEQ), 10, '0', STR_PAD_LEFT);
            $MOVCAIXADLV_SEQ++;
            $this->orderFactory->insertChange($orderData, $moneyReceipt, $generalParams, $consumerDetails, $movCaixa);
        }

        $generalParams["NRSEQPRODCUP"] = $this->util->newCode("ITCOMANDAVEN".$generalParams["CDFILIAL"], 10);
        
        $products         = [];
        $ITCOMANDAVEN_SEQ = 1;
        
        if(!empty($orderData["ItemOrder"])) {
            foreach ($orderData["ItemOrder"] as $product) {
                $products = array_merge($products, $this->productService->processProducts($product, $orderData, $generalParams, $ITCOMANDAVEN_SEQ));
            }
        }
        
        //Produto Combinado
        if(!empty($orderData['CombinedProducts'])) {
            $this->productService->clearCombinedProducts();
            
            foreach($orderData['CombinedProducts'] as $combinedProduct) {
                if($combinedProduct["price"] > 0)
                    throw new Exception\ParamsException("Envio do produto combinado inválido. O produto '{$combinedProduct['title']}' deve ter o preço ser zerado.");
                
                if(!$this->isCombinedProduct($combinedProduct['combination']))
                    throw new Exception\ParamsException("Envio do produto combinado inválido. Nas combinações deve possuir ao menos um item com a inicial 'Massa'.");
                
                $products = array_merge($products, $this->productService->processCombinedProducts($combinedProduct, $orderData, $generalParams, $ITCOMANDAVEN_SEQ));
            }
        }
        
        if(!empty($orderData['discount'])) 
            $this->productService->shareDiscount($orderData['discount'], $products);

        if(empty($products)) {
            throw new \Exception("Falha ao processar os produtos do pedido.");
        }

        foreach($products as $product) {
            $this->productService->insertProduct($product, $generalParams, $orderData);
            
            //Observações da OBSITCOMANDAVEN
            if(!empty($product['ITEM_OBSERVATION'])) {
                foreach($product['ITEM_OBSERVATION'] as $itemObservation) {
                    $this->productService->insertObservation($itemObservation, $generalParams, $orderData);
                }                
            }
        }
        
        // orderId Hash
        // CDFILIAL-NRCOMANDA-NRVENDAREST-NRCOMANDAEXT
        return "{$generalParams['CDFILIAL']}-{$orderData['NRCOMANDA']}-{$orderData['NRVENDAREST']}-{$orderData['orderNumber']}";
    }
    
    private function isCombinedProduct($subItems) {
        foreach($subItems as $item) {
            if(strtoupper(substr($item['title'], 0, 5)) === 'MASSA') {
                return true;
            }
        }
        return false;
    }
    
    public function getSalesWithoutOrigin() {
        return $this->orderFactory->getSalesWithoutOrigin();
    }
    
    public function setSalesOrigin($salesData) {
        foreach ($salesData as $item) { 
            $this->orderFactory->setSaleOrigin($item);
        }
    }
    
    public function orderCancel($orderHash) {
        $orderData = explode("-", $orderHash);
        if(count($orderData) != 4) {
            throw new \Exception("Identificador de pedido inválido.");
        }

        $cdFilial     = $orderData[0];
        $nrComanda    = $orderData[1];
        $nrVendaRest  = $orderData[2];
        $nrComandaExt = $orderData[3];

        return $this->orderFactory->orderCancel($cdFilial, $nrComanda, $nrVendaRest, $nrComandaExt);
    }
    
    public function getOrderStatus($orderHash) {
        $statusList = [
            1   => 'PRODUCAO',
            2   => 'PRODUCAO',
            3   => 'PRODUCAO',
            4   => 'CANCELADO',
            5   => 'EXPEDIDO',
            'X' => 'FINALIZADO',
            'P' => 'IMPRESSO'
        ];
        
        $orderData = explode("-", $orderHash);
        if(count($orderData) != 4) {
            throw new \Exception("Identificador de pedido inválido.");
        }

        $cdFilial     = $orderData[0];
        $nrComanda    = $orderData[1];
        $nrVendaRest  = $orderData[2];
        $nrComandaExt = $orderData[3];

        $currentOrderStatus = $this->orderFactory->getOrderStatus($cdFilial, $nrComanda, $nrVendaRest, $nrComandaExt);
        return isset($statusList[$currentOrderStatus['IDSTCOMANDA']]) ? $statusList[$currentOrderStatus['IDSTCOMANDA']] : "DESCONHECIDO";
    }
    
    // Função de validação de parametros para venda credito pessoal
    public function validatePersonalCredit($orderData, $consumerDetails) {
        $generalParams["CDFILIAL"] = $orderData["subsidiaryCode"];
        
        if (empty($orderData["consumerDetail"]["consumerRef"])) {
             throw new Exception\ParamsException("Venda crédito pessoal sem informação de identificação do consumidor.");
        }
        $generalParams["CDCONSUMIDOR"] = $consumerDetails["CDCONSUMIDOR"];
        $generalParams["CDCLIENTE"] = $consumerDetails["CDCLIENTE"];
        $PendingSale = $this->orderFactory->getPendingSaleConsumer($generalParams, $orderData); 
        if (!empty($PendingSale)){
             throw new Exception\ParamsException("Existem comanda(s) em aberto com o tipo de recebimento de credito pessoal para o consumidor: {$consumerDetails['NMCONSUMIDOR']} - filial: {$PendingSale['0']['CDFILIAL']} - valor: {$PendingSale['0']['VRMOVIVENDDLV']}");
        }

        $arraytotalfamilia = [];
        $arrayValida = [];
        
        foreach ($orderData["ItemOrder"] as $product) {
            $productDetails = $this->productFactory->getProduct($product["ref"]);
            
            if (empty($productDetails))
                throw new Exception\ParamsException("Não foi encontrado nenhum produto para o código de integração: {$product['ref']} - Produto: {$product['title']}");
            $generalParams = array_merge($productDetails, $generalParams);
            $productFamilyDetails = $this->productFactory->getProductFamilyDetails($generalParams);
            if (empty($productFamilyDetails))
                throw new Exception\ParamsException("Não foi encontrado familia de produto para o produto. Código de integração: {$product['ref']} - Produto: {$product['title']}");
            $product["CDFAMILISALD"] = $productFamilyDetails[0]["CDFAMILISALD"];
            
            if (isset($arraytotalfamilia[$product["CDFAMILISALD"]]))
                $arraytotalfamilia[$product["CDFAMILISALD"]] += $product["total"];
            else    
                $arraytotalfamilia[$product["CDFAMILISALD"]] = $product["total"];
                
        }
        
        foreach($arraytotalfamilia as $key => $line) {
            $generalParams["CDFAMILISALD"] = $key;
            $result = $this->orderFactory->getPersonalCreditValue($generalParams);
            
            if (empty($result))
                throw new Exception\ParamsException("Familia de produto: {$generalParams['CDFAMILISALD']} não encontrada para o consumidor: {$consumerDetails['NMCONSUMIDOR']}");
            
            foreach($result as $index => $resultline) {
                if ($resultline["CDFAMILISALD"] == $key) {
                    $resultline["VRSALDCONFAM"] >= $line ? true : array_push($arrayValida, $resultline["NMFAMILISALD"]);
                }
            }
        }
        $resp = '';
        foreach ($arrayValida as $index => $familyname) {
            if ($index == 0)
                $resp = $familyname;
            else 
                $resp +=', '. $familyname;
        }
        
        if (count($arrayValida) == 0) {
            return null;   
        }
        return $resp;
        
        
    }
    
    public function getCdSubProj($origin){
        return $this->orderFactory->getCdSubProj($origin);
    }
    
    public function getSubProjectUrlOrders($origin){
        return $this->orderFactory->getSubProjectUrlOrders($origin);
    }
    
    public function getSubProjectUrlSales($origin){
        return $this->orderFactory->getSubProjectUrlSales($origin);
    }
    
    public function getOrderHistory($cdSubProj){
        return $this->orderFactory->getOrderHistory($cdSubProj);
    }
    
    public function getSalesWithNfceAccess($cdSubProj){
        return $this->orderFactory->getSalesWithNfceAccess($cdSubProj);
    }
    
    public function insertOrderHistControl($order, $IDSTATUSENVIO, $response, $cdSubProj){
        return $this->orderFactory->insertOrderHistControl($order, $IDSTATUSENVIO, $response, $cdSubProj);
    }
    
    public function insertSaleAcessNfceControl($sale, $IDSTATUSENVIO, $response, $cdSubProj){
        return $this->orderFactory->insertSaleAcessNfceControl($sale, $IDSTATUSENVIO, $response, $cdSubProj);
    }
}
